/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : Flash_Programming.c
* $Rev: $
* $Date::                           $
* Description : Initialize peripheral function sample
*******************************************************************************/


/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "r_typedefs.h"
#include "iodefine.h"
#include "rza_io_regrw.h"
#include "Flash.h"

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/
#define FLASH_BUFFER_SIZE        (256 * 1024)


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
void RZ_A1H_FlashProgram(int argc, char **argv);


/******************************************************************************
Private global variables and functions
******************************************************************************/
static int_t RZ_A1H_FlashProgram_Sub(int argc, char **argv);

static uint8_t g_flash_buffer[FLASH_BUFFER_SIZE];
static char full_path[256+1];

/******************************************************************************
* Function Name: RZ_A1H_FlashProgram
* Description  : Flash programming function
* Arguments    : int argc
*              : char **argv
* Return Value : none
******************************************************************************/
void RZ_A1H_FlashProgram(int argc, char **argv)
{

    printf("Start to load Binary Data to Flash Memory? [y or n]:");
    scanf("%s", g_flash_buffer);
    if( (g_flash_buffer[0] != 'y') && (g_flash_buffer[0] != 'Y') )
    {
        printf("Flash Programming Cancel\n");
        return;
    }

    printf("Initializing Flash...\n");
    if( Flash_Write_Init() != 0 )
    {
        printf("error to change s-flash mode \n");
        return;
    }


    if( atoi(argv[2]) == 2)
    {
        printf("Data is programmed to sflash(x 2) \n");
        if( Flash_Write_Init_Dual() != 0 )
        {
            printf("error to change dual s-flash mode \n");
            return;
        }
    }
    else
    {
       printf("Data is programmed to sflash(x 1) \n");

    }

    RZ_A1H_FlashProgram_Sub(argc, argv);
}

/******************************************************************************
* Function Name: RZ_A1H_FlashProgram_Sub
* Description  : main function
* Arguments    : int argc
*              : char **argv
* Return Value : result to program flash
******************************************************************************/
static int_t RZ_A1H_FlashProgram_Sub(int argc, char **argv)
{
    FILE            *srcfile = 0;
    uint32_t        address;
    uint32_t        *nextaddress;
    uint32_t        *temp;
    int_t           bytesread;
    int_t           filesize;
    int_t           cur_arg_no = 3;

    while( cur_arg_no < argc )
    {

        if( Flash_Write_Entry() != 0 )
        {
            printf("error to change to SPI mode.\n");
            return -1;
        }

        full_path[0] ='.';
        full_path[1] ='\\';
        full_path[2] ='\0';
        strcat(full_path, argv[1]);


        strcat(full_path,"\\");
        strcat(full_path,argv[cur_arg_no]);


        printf("\nfilename=%s \n", full_path);


        sscanf( argv[cur_arg_no+1], "%x", &address );
        if(address >= 0x1C000000)
        {
            printf("address error \n");
            return -1;
        }
        if(atoi(argv[2]) == 2) /* sflash x 2 */
        {
            if( ((address) % (SF_PAGE_SIZE * 2)) !=0 )
            {
                printf("address error \n");
                return -1;
            }
        }
        else /* sflash x 1 */
        {
            if( (address % SF_PAGE_SIZE) !=0 )
            {
                printf("address error \n");
                return -1;
            }
        }


        printf( "addr=0x%08X\n", address );

        srcfile = fopen(full_path, "rb");
        if( srcfile == 0 )
        {
            printf("Could not open file. Exiting.\n");
            return -1;
        }
        



        if( fseek(srcfile,0,SEEK_END) != 0 )
        {
            printf("file seek error \n");
        }
        filesize = ftell(srcfile);
        if( filesize <= 0 )
        {
            printf("file  error \n");
        }
        
        printf("Data Size is %d\n", filesize);
        fseek( srcfile, 0L, SEEK_SET );
        
        printf("Programming Flash...\n");
        
        /* ==== set the flash address ==== */
        nextaddress = (uint32_t *)address;
        
        /* ==== read data from file ==== */
        bytesread   = fread( &g_flash_buffer[0], sizeof(uint8_t), FLASH_BUFFER_SIZE, srcfile );
        
        while( bytesread != 0 )
        {
            /* ==== write data ==== */
            if( Flash_Write_Area( nextaddress, (uint32_t *)&g_flash_buffer[0], bytesread ) != 0 )
            {
                printf("Programing Error %x\n", (int_t)nextaddress);
                fclose( srcfile );
                return -1;
            }
            
            /* ==== next data ==== */
            nextaddress += (bytesread / sizeof(uint32_t));
            bytesread    = fread( &g_flash_buffer[0], sizeof(uint8_t), FLASH_BUFFER_SIZE, srcfile );
        }
        
        printf("Verifying Flash...\n");
        fseek( srcfile, 0, SEEK_SET );

        if(Flash_Veify_Entry() != 0)
        {
            printf("error to change to SPIBSC mode.\n");
            return -1;
        }
        
        /* ==== set the flash address ==== */
        nextaddress = (uint32_t *)address;
        
        /* ==== read data from file ==== */
        bytesread   = fread( &g_flash_buffer[0], sizeof(uint8_t), FLASH_BUFFER_SIZE, srcfile );
        while( bytesread != 0 )
        {
            temp = (uint32_t *)&g_flash_buffer[0];
            while( bytesread != 0 )
            {
                if( bytesread < sizeof(uint32_t) )
                {
                    if( bytesread == 3 )
                    {
                        if( (*nextaddress & 0x00ffffff) != (*temp & 0x00ffffff) )
                        {
                            printf("Verify failed.  Exiting. 0x%x\n", (int_t)nextaddress);
                            fclose( srcfile );
                            return -1;
                        }
                    }
                    if( bytesread == 2 )
                    {
                        if( (*nextaddress & 0x0000ffff) != (*temp & 0x0000ffff) )
                        {
                            printf("Verify failed.  Exiting. 0x%x\n", (int_t)nextaddress);
                            fclose( srcfile );
                            return -1;
                        }
                    }
                    if( bytesread == 1 )
                    {
                        if( (*nextaddress & 0x000000ff) != (*temp & 0x000000ff) )
                        {
                            printf("Verify failed.  Exiting. 0x%x\n", (int_t)nextaddress);
                            fclose( srcfile );
                            return -1;
                        }
                    }
                    
                    fclose( srcfile );
                    return 0;
                }
                
                if( *nextaddress != *temp )
                {
                    printf("Verify failed.  Exiting. 0x%x\n", (int_t)nextaddress);
                    fclose( srcfile );
                    return -1;
                }
                
                nextaddress++;
                temp++;
                bytesread -= sizeof(uint32_t);
            }
            
            bytesread = fread( &g_flash_buffer[0], sizeof(uint8_t), FLASH_BUFFER_SIZE, srcfile );
        }
        
        fclose( srcfile );

        cur_arg_no += 2;
    }
    
    return 0;
}

/* End of File */
